if (sfeDoNotLoad) then
	return;
end

local function IsFirstCharNarrow(text)
	if (not text) then
		return false;
	end
	local firstChar = string.sub(text, 1, 1);
	return firstChar == "1"
		or firstChar == "i"
		or firstChar == "I"
		or firstChar == "l"
		or firstChar == ""
		or firstChar == "!";
end

local QuestTooltipScanner = CreateFrame("GameTooltip", "QuestTooltipScanner", nil, "GameTooltipTemplate");
local function IsQuestItem(itemId)
	if (not itemId) then
		return nil;
	end
	QuestTooltipScanner:SetOwner(UIParent, "ANCHOR_NONE");
	QuestTooltipScanner:SetItemByID(itemId);
	QuestTooltipScanner:Show();
	local i;
	for i = 1, QuestTooltipScanner:NumLines() do
		if (_G["QuestTooltipScannerTextLeft"..i]:GetText() == ITEM_STARTS_QUEST) then
			return true;
		end
	end
	QuestTooltipScanner:Hide();
	return false;
end

function Sfe:UpdateOrderHallMissionFrame()
	local missionList = OrderHallMissionFrame.MissionTab.MissionList;

	if (SFE_ORDERHALL_MISSIONS_ENHANCED) then
		missionList.listScroll:SetPoint("TOPLEFT", missionList, "TOPLEFT", 0, -6);
		missionList.listScroll:SetPoint("BOTTOMRIGHT", missionList, "BOTTOMRIGHT", -3, 7);
		missionList.listScroll.scrollBar:SetPoint("TOPLEFT", missionList.listScroll, "TOPRIGHT", -20, -18);
		missionList.listScroll.scrollBar:SetPoint("BOTTOMLEFT", missionList.listScroll, "BOTTOMRIGHT", -20, 13);
		missionList.listScroll.buttonHeight = 79 + (16.0 / #missionList.listScroll.buttons); -- Shame
		missionList.Tab1:SetFrameLevel(7);
		missionList.Tab2:SetFrameLevel(7);
		missionList.Border:Show();
	else
		missionList.listScroll:SetPoint("TOPLEFT", missionList, "TOPLEFT", 5, -6);
		missionList.listScroll:SetPoint("BOTTOMRIGHT", missionList, "BOTTOMRIGHT", -3, 8);
		missionList.listScroll.scrollBar:SetPoint("TOPLEFT", missionList.listScroll, "TOPRIGHT", -20, -12);
		missionList.listScroll.scrollBar:SetPoint("BOTTOMLEFT", missionList.listScroll, "BOTTOMRIGHT", -20, 12);
		missionList.listScroll.buttonHeight = 79;
		missionList.Tab1:SetFrameLevel(3);
		missionList.Tab2:SetFrameLevel(3);
		missionList.Border:Hide();
	end
end

function Sfe:OrderHallMissionFrame_MissionTab_MissionListPostHook(self)
	if (not self or not self:IsShown()) then
		return;
	end

	local missions = self.showInProgress and self.inProgressMissions or self.availableMissions;
	local buttons = self.listScroll.buttons;
	local offset = HybridScrollFrame_GetOffset(self.listScroll);

	if (SFE_ORDERHALL_MISSIONS_ENHANCED) then
		for i = 1, #buttons do
			local button = buttons[i];
			button:SetWidth(846);
			local index = offset + i;

			if (index <= #missions) then
				local mission = missions[index];

				-- Time details
				if (not mission.inProgress) then
					local durationText = string.format(PARENS_TEMPLATE, (mission.durationSeconds < GARRISON_LONG_MISSION_TIME) and mission.duration or string.format(GARRISON_LONG_MISSION_TIME_FORMAT, mission.duration));
					local durationTextLength = string.len(durationText);
					if (string.sub(durationText, durationTextLength) == ")") then
						local expirationMessage = "";
						if (mission.offerEndTime) then
							local remainingSeconds = mission.offerEndTime - GetTime();
							local colorCode, colorCodeEnd = "", "";
							if (remainingSeconds < 8 * 3600) then
								colorCode, colorCodeEnd = RED_FONT_COLOR_CODE, FONT_COLOR_CODE_CLOSE;
							elseif (remainingSeconds < 24 * 3600) then
								colorCode, colorCodeEnd = YELLOW_FONT_COLOR_CODE, FONT_COLOR_CODE_CLOSE;
							end
							expirationMessage = ", "..colorCode..string.format(SFE_EXPIRES_IN, mission.offerTimeRemaining)..colorCodeEnd;
						end
						local slowingMessage = "";
						local _, _, _, _, _, _, _, enemies = C_Garrison.GetMissionInfo(mission.missionID);
						if (enemies) then
							for _, enemy in ipairs(enemies) do
								if (enemy.mechanics) then
									for _, mechanic in pairs(enemy.mechanics) do
										if (mechanic.ability and mechanic.ability.id and mechanic.ability.id == 428) then
											slowingMessage = slowingMessage.."|TInterface\\Garrison\\orderhall-missions-mechanic5:0|t"
										end
									end
								end
							end
							if (slowingMessage ~= "") then
								slowingMessage = " "..slowingMessage;
							end
						end
						button.Summary:SetText(string.sub(durationText, 2, durationTextLength - 1)..slowingMessage..expirationMessage);
					end
				end
				button.Title:SetPoint("LEFT", 165, 10);
				button.Title:SetWidth(655 - #mission.rewards * 65);
				button.Summary:ClearAllPoints();
				button.Summary:SetPoint("TOPLEFT", button.Title, "BOTTOMLEFT", 0, -4);

				-- Level
				local isMaxLevel = mission.isMaxLevel and mission.iLevel > 0;
				if (isMaxLevel) then
					button.Level:SetText(mission.iLevel);
					button.ItemLevel:SetText(ITEM_LEVEL_ABBR);
				end
				-- WoW calculates fonts as monospaced, which visually is false, since '1' is thin it doesn't look centered
				local isLevelTextFirstCharNarrow = IsFirstCharNarrow(button.Level:GetText());
				button.Level:SetPoint("CENTER", button, "LEFT", isLevelTextFirstCharNarrow and 37 or 40, isMaxLevel and (- button.ItemLevel:GetHeight() / 2) or 0);
				button.ItemLevel:ClearAllPoints();
				button.ItemLevel:SetPoint("BOTTOM", button.Level, "TOP", (isLevelTextFirstCharNarrow and 3 or 0) - ((IsFirstCharNarrow(button.ItemLevel:GetText())) and 2 or 0), 2);

				-- Rarity
				if (mission.isRare) then
					button.Level:SetTextColor(0.098, 0.537, 0.969, 1.0);
					button.ItemLevel:SetTextColor(0.098, 0.537, 0.969, 1.0);
				else
					button.Level:SetTextColor(0.84, 0.72, 0.57, 1.0);
					button.ItemLevel:SetTextColor(0.84, 0.72, 0.57, 1.0);
				end
				button.RareText:Hide();

				-- Type
				button.MissionType:SetPoint("TOPLEFT", button, "TOPLEFT", 142, -37);
				if (OrderHallMissionFrame.followerTypeID == LE_FOLLOWER_TYPE_GARRISON_7_0) then
					if (mission.typeAtlas == "ClassHall-BonusIcon-Desaturated") then
						button.MissionType:SetSize(71, 71);
						button.MissionType:SetPoint("TOPLEFT", 68, -2);
						if (button.MissionType.TreasureOverlay) then
							button.MissionType.TreasureOverlay:Hide();
						end
					elseif (mission.typeAtlas == "ClassHall-TreasureIcon-Desaturated") then
						button.MissionType:SetSize(62, 62);
						button.MissionType:SetPoint("TOPLEFT", 74, -6);
						if (not button.MissionType.TreasureOverlay) then
							button.MissionType.TreasureOverlay = button:CreateTexture(nil, "ATRWORK", nil, 2);
							button.MissionType.TreasureOverlay:SetTexture("Interface\\AddOns\\StandardFramesEnhanced_OrderHall\\images\\OrderHallMissionTypeTreasureIconOverlay");
							button.MissionType.TreasureOverlay:SetSize(92, 92);
							button.MissionType.TreasureOverlay:SetPoint("CENTER", button.MissionType);
						end
						button.MissionType.TreasureOverlay:Show()
					else
						button.MissionType:SetSize(62, 62);
						button.MissionType:SetPoint("TOPLEFT", 74, -6);
						if (button.MissionType.TreasureOverlay) then
							button.MissionType.TreasureOverlay:Hide();
						end
					end
				else
					button.MissionType:SetSize(75, 75);
					button.MissionType:SetPoint("TOPLEFT", 68, -2);
				end

				-- Rewards
				local allMissionRewards = {};
				if (mission.rewards) then
					for j = 1, #mission.rewards do
						allMissionRewards[#allMissionRewards + 1] = mission.rewards[j];
					end
				end
				if (mission.overmaxRewards) then
					for j = 1, #mission.overmaxRewards do
						allMissionRewards[#allMissionRewards + 1] = mission.overmaxRewards[j];
					end
				end
				if (button.Title:GetWidth() > 655 - 12 - #allMissionRewards * 56) then
					button.Title:SetWidth(655 - 12 - #allMissionRewards * 56);
				end
				if (button.Summary:GetWidth() > 655 - 12 - #allMissionRewards * 56) then
					button.Summary:SetWidth(655 - 12 - #allMissionRewards * 56);
				end
				GarrisonMissionButton_SetRewards(button, allMissionRewards);
				for j = 1, #button.Rewards do
					button.Rewards[j]:SetSize(48, 48);
					if (j <= #allMissionRewards) then
						button.Rewards[j]:Show();
					else
						button.Rewards[j]:Hide();
					end
					if (j ~= 1) then
						button.Rewards[j]:SetPoint("RIGHT", button.Rewards[j - 1], "LEFT", 48 + 8 + 48, 0);
					else
						button.Rewards[j]:SetPoint("RIGHT", button, "RIGHT", -12 - (8 + 48) * (#allMissionRewards - 1), 0);
					end
					button.Rewards[j].Icon:SetMask("Interface\\ICONS\\INV_Helmet_47");
					if (not button.Rewards[j].QuestIcon) then
						button.Rewards[j].QuestIcon = button.Rewards[j]:CreateTexture(nil, "ARTWORK", nil, 2);
						button.Rewards[j].QuestIcon:SetTexture(TEXTURE_ITEM_QUEST_BANG);
						button.Rewards[j].QuestIcon:SetPoint("TOPLEFT", button.Rewards[j], "TOPLEFT", 1, 0);
						button.Rewards[j].QuestIcon:SetPoint("BOTTOMRIGHT", button.Rewards[j], "BOTTOMRIGHT", 0, 1);
					end
					if (IsQuestItem(button.Rewards[j].itemID)) then
						button.Rewards[j].QuestIcon:Show();
						button.Rewards[j].IconBorder:SetDrawLayer("ARTWORK", 3);
					else
						button.Rewards[j].QuestIcon:Hide();
					end
					if (not button.Rewards[j].BonusIcon) then
						button.Rewards[j].BonusIcon = CreateFrame("Frame", nil, button);
						button.Rewards[j].BonusIcon:SetSize(28, 28);
						button.Rewards[j].BonusIcon:SetPoint("CENTER", button.Rewards[j], "TOPRIGHT", -8, -8);
						button.Rewards[j].BonusIcon:SetHitRectInsets(4, 4, 1, 4);
						button.Rewards[j].BonusIcon.texture = button.Rewards[j].BonusIcon:CreateTexture(nil, "ARTWORK", nil, 2);
						button.Rewards[j].BonusIcon.texture:SetAtlas("honorsystem-icon-bonus");
						button.Rewards[j].BonusIcon.texture:SetSize(28, 28);
						button.Rewards[j].BonusIcon.texture:SetPoint("CENTER", button.Rewards[j].BonusIcon, "CENTER", 0, 0);
						button.Rewards[j].BonusIcon:SetScript("OnEnter", function(self, motion)
							GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
							GameTooltip:SetText(BONUS_REWARDS, nil, nil, nil, nil, true);
						end);
						button.Rewards[j].BonusIcon:SetScript("OnLeave", GameTooltip_Hide);
					end
					if (not button.Rewards[j].ChanceText) then
						button.Rewards[j].ChanceText = button.Rewards[j]:CreateFontString(nil, "ARTWORK", "Game15Font_o1");
						button.Rewards[j].ChanceText:SetPoint("CENTER", button.Rewards[j], "CENTER", 0, -1);
						button.Rewards[j].ChanceText:SetTextColor(0.84, 0.72, 0.5, 0.9);
					end
					local chanceOfSuccess = C_Garrison.GetMissionSuccessChance(mission.missionID);
					if (#mission.rewards < j) and (j <= #allMissionRewards) then
						button.Rewards[j].BonusIcon:Show();
						button.Rewards[j].ChanceText:SetFormattedText(PERCENTAGE_STRING, math.max(chanceOfSuccess - 100, 0));
					else
						button.Rewards[j].BonusIcon:Hide();
						button.Rewards[j].ChanceText:SetFormattedText(PERCENTAGE_STRING, math.min(chanceOfSuccess, 100));
					end
					if (mission.inProgress) then
						button.Rewards[j].ChanceText:Show();
					else
						button.Rewards[j].ChanceText:Hide();
					end
				end

				-- Hovering
--				local hoveringAnimationDuration = 1.0; -- TODO Make it constant
--				local fromAlpha, toAlpha = 0.5, 0.9;
--				if (not button.HoveringAnimationGroup) then
--					button.HoveringAnimationGroup = button:CreateAnimationGroup();
--					button.HoveringAnimationGroup:SetToFinalAlpha(false);
--					button.HoveringAnimationGroup:SetLooping("BOUNCE");
--					button.HoveringAnimationGroup:SetScript("OnLoop", function(self, loopState)
----						self:Pause();
--						print(loopState);
--					end);
--				end
--				for j = 1, #button.Rewards do
--					if (not button.Rewards[j].HoveringAnimation) then
--						button.Rewards[j].HoveringAnimation = button.HoveringAnimationGroup:CreateAnimation("Alpha");
--						button.Rewards[j].HoveringAnimation:SetOrder(1);
--						button.Rewards[j].HoveringAnimation:SetTarget(button.Rewards[j].ChanceText);
--						button.Rewards[j].HoveringAnimation:SetFromAlpha(fromAlpha);
--						button.Rewards[j].HoveringAnimation:SetToAlpha(toAlpha);
--						button.Rewards[j].HoveringAnimation:SetSmoothing("IN_OUT");
--						button.Rewards[j].HoveringAnimation:SetDuration(hoveringAnimationDuration);
--					end
--				end
--				if (not button.originalOnEnter) then
--					button.originalOnEnter = button:GetScript("OnEnter");
--					button:SetScript("OnEnter", function(self, motion)
--						button.originalOnEnter(self, motion);
--						if (button.HoveringAnimationGroup) then
----							if (button.HoveringAnimationGroup:IsPendingFinish()) then
----								button.HoveringAnimationGroup:Pause();
----							end
----							local i;
----							for i = 1, #button.Rewards do
----								if (button.Rewards[i].HoveringAnimation) then
----									local currentAlpha = button.Rewards[i].ChanceText:GetAlpha();
----									button.Rewards[i].HoveringAnimation:SetFromAlpha(currentAlpha);
----									button.Rewards[i].HoveringAnimation:SetToAlpha(toAlpha);
----									button.Rewards[i].HoveringAnimation:SetDuration((toAlpha - (currentAlpha - fromAlpha)) * hoveringAnimationDuration);
----									print(fromAlpha.." "..currentAlpha.." "..toAlpha.." "..button.Rewards[i].HoveringAnimation:GetDuration());
----								end
----							end
--							button.HoveringAnimationGroup:Play();
--						end
--					end);
--				end
--				if (not button.originalOnLeave) then
--					button.originalOnLeave = button:GetScript("OnLeave");
--					button:SetScript("OnLeave", function(self, motion)
--						button.originalOnLeave(self, motion);
--						if (button.HoveringAnimationGroup) then
----							if (button.HoveringAnimationGroup:IsPendingFinish()) then
----								button.HoveringAnimationGroup:Pause();
----							end
----							local i;
----							for i = 1, #button.Rewards do
----								if (button.Rewards[i].HoveringAnimation) then
----									local currentAlpha = button.Rewards[i].ChanceText:GetAlpha();
----									button.Rewards[i].HoveringAnimation:SetFromAlpha(currentAlpha);
----									button.Rewards[i].HoveringAnimation:SetToAlpha(fromAlpha);
----									button.Rewards[i].HoveringAnimation:SetDuration((-(fromAlpha - (currentAlpha - toAlpha))) * hoveringAnimationDuration);
----									print(toAlpha.." "..currentAlpha.." "..fromAlpha.." "..button.Rewards[i].HoveringAnimation:GetDuration());
----								end
----							end
--							button.HoveringAnimationGroup:Pause();
--						end
--					end);
--				end
			end
		end
	else
		for i = 1, #buttons do
			local button = buttons[i];
			button:SetWidth(832);

			if (i + offset <= #missions) then
				local mission = missions[i + offset];

				-- Level
				if (mission.isRare and mission.isMaxLevel and mission.iLevel > 0) then
					button.Level:SetPoint("CENTER", button, "TOPLEFT", 40, -22);
				else
					button.Level:SetPoint("CENTER", button, "TOPLEFT", 40, -36);
				end
				if (mission.isRare) then
					button.RareText:Show();
				end
				button.ItemLevel:ClearAllPoints();
				button.ItemLevel:SetPoint("TOP", button.Level, "BOTTOM", 0, 2);

				-- Rarity
				button.Level:SetTextColor(0.84, 0.72, 0.57, 1.0);
				button.ItemLevel:SetTextColor(0.84, 0.72, 0.57, 1.0);

				-- Type
				button.MissionType:SetPoint("TOPLEFT", button, "TOPLEFT", 68, -2);
				if (OrderHallMissionFrame.followerTypeID == LE_FOLLOWER_TYPE_GARRISON_7_0) then
					button.MissionType:SetSize(62, 62);
					button.MissionType:SetPoint("TOPLEFT", 74, -6);
				else
					button.MissionType:SetSize(75, 75);
					button.MissionType:SetPoint("TOPLEFT", 68, -2);
				end
				if (button.MissionType.TreasureOverlay) then
					button.MissionType.TreasureOverlay:Hide();
				end

				-- Rewards
				for j = 1, #button.Rewards do
					button.Rewards[j]:SetSize(72, 72);
					if (j ~= 1) then
						button.Rewards[j]:SetPoint("RIGHT", button.Rewards[j - 1], "LEFT", 0, 0);
					else
						button.Rewards[j]:SetPoint("RIGHT", button, "RIGHT", 0, 0);
					end
					button.Rewards[j].Icon:SetMask(nil);
					button.Rewards[j].IconBorder:SetDrawLayer("ARTWORK", 1);
					if (button.Rewards[j].QuestIcon) then
						button.Rewards[j].QuestIcon:Hide();
					end
					if (button.Rewards[j].BonusIcon) then
						button.Rewards[j].BonusIcon:Hide();
					end
					if (button.Rewards[j].ChanceText) then
						button.Rewards[j].ChanceText:Hide();
					end
				end

				-- Hovering
--				if (button.HoveringAnimationGroup) then
--					button.HoveringAnimationGroup:Stop();
--				end
--				if (button.originalOnEnter) then
--					button:SetScript("OnEnter", button.originalOnEnter);
--					button.originalOnEnter = nil;
--				end
--				if (button.originalOnLeave) then
--					button:SetScript("OnLeave", button.originalOnLeave);
--					button.originalOnLeave = nil;
--				end
			end
		end
	end
end

local eventController = CreateFrame("Frame");
eventController:RegisterEvent("ADDON_LOADED");
eventController:SetScript("OnEvent", function(self, event, name)
		if ((event == "ADDON_LOADED") and (name == "StandardFramesEnhanced_OrderHall")) then
			if (SFE_ORDERHALL_MISSIONS_ENHANCED == nil) then
				SFE_ORDERHALL_MISSIONS_ENHANCED = true;
			end

			local missionList = OrderHallMissionFrame.MissionTab.MissionList;

			if (OrderHallMissionFrame.EnhancedCheckbox == nil) then
				OrderHallMissionFrame.EnhancedCheckbox = CreateFrame("CheckButton", "OrderHallMissionFrameEnhancedCheckbox", OrderHallMissionFrame.MissionTab, "OptionsBaseCheckButtonTemplate");
				OrderHallMissionFrame.EnhancedCheckbox:SetHitRectInsets(0, 0, 0, 0);

				OrderHallMissionFrame.EnhancedCheckbox:SetPoint("RIGHT", OrderHallMissionFrame.CloseButton, "LEFT", 7, 0);
				OrderHallMissionFrame.EnhancedCheckbox:SetAlpha(0.1);
				OrderHallMissionFrame.EnhancedCheckbox:SetScript("OnClick", function(self, button, down)
						SFE_ORDERHALL_MISSIONS_ENHANCED = self:GetChecked();
						Sfe:UpdateOrderHallMissionFrame();
						missionList:Update();
						if (SFE_ORDERHALL_MISSIONS_ENHANCED) then
							PlaySound("igMainMenuOptionCheckBoxOn");
						else
							PlaySound("igMainMenuOptionCheckBoxOff");
						end
					end);
				OrderHallMissionFrame.EnhancedCheckbox:SetScript("OnEnter", function(self, motion)
						self:SetAlpha(1.0);
						GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
						GameTooltip:SetText(SFE_ENHANCED_UI, nil, nil, nil, nil, true);
					end);
				OrderHallMissionFrame.EnhancedCheckbox:SetScript("OnLeave", function(self, motion)
						self:SetAlpha(0.1);
						GameTooltip_Hide(self, motion);
					end);
			end
			OrderHallMissionFrame.EnhancedCheckbox:SetChecked(SFE_ORDERHALL_MISSIONS_ENHANCED);

			missionList.Border = CreateFrame("Frame", nil, missionList.listScroll);
			missionList.Border:SetPoint("TOPLEFT", missionList.listScroll, "TOPLEFT", 0, 6);
			missionList.Border:SetPoint("BOTTOMRIGHT", missionList.listScroll, "BOTTOMRIGHT", 3, -7);
			local tex;
			tex = missionList.Border:CreateTexture(nil, "BORDER", 4);
			tex:SetAtlas("_Garr_InfoBoxBorderMission-Top", true);
			tex:SetHorizTile(true);
			tex:SetPoint("TOPLEFT");
			tex:SetPoint("TOPRIGHT");
			tex = missionList.Border:CreateTexture(nil, "BORDER", 4);
			tex:SetAtlas("_Garr_InfoBoxBorderMission-Top", true);
			tex:SetHorizTile(true);
			tex:SetTexCoord(0, 1, 1, 0);
			tex:SetPoint("BOTTOMLEFT");
			tex:SetPoint("BOTTOMRIGHT");
			tex = missionList.Border:CreateTexture(nil, "BORDER", 4);
			tex:SetAtlas("!Garr_InfoBoxBorderMission-Left", true);
			tex:SetVertTile(true);
			tex:SetPoint("TOPLEFT");
			tex:SetPoint("BOTTOMLEFT");
			tex = missionList.Border:CreateTexture(nil, "BORDER", 4);
			tex:SetAtlas("!Garr_InfoBoxBorderMission-Left", true);
			tex:SetVertTile(true);
			tex:SetTexCoord(1, 0, 0, 1);
			tex:SetPoint("TOPRIGHT");
			tex:SetPoint("BOTTOMRIGHT");
			tex = missionList.Border:CreateTexture(nil, "BORDER", 7);
			tex:SetAtlas("Garr_InfoBoxBorderMission-Corner", true);
			tex:SetPoint("TOPLEFT");
			tex = missionList.Border:CreateTexture(nil, "BORDER", 7);
			tex:SetAtlas("Garr_InfoBoxBorderMission-Corner", true);
			tex:SetTexCoord(1, 0, 0, 1);
			tex:SetPoint("TOPRIGHT");
			tex = missionList.Border:CreateTexture(nil, "BORDER", 7);
			tex:SetAtlas("Garr_InfoBoxBorderMission-Corner", true);
			tex:SetTexCoord(0, 1, 1, 0);
			tex:SetPoint("BOTTOMLEFT");
			tex = missionList.Border:CreateTexture(nil, "BORDER", 7);
			tex:SetAtlas("Garr_InfoBoxBorderMission-Corner", true);
			tex:SetTexCoord(1, 0, 1, 0);
			tex:SetPoint("BOTTOMRIGHT");
			missionList.Border:Hide();

			Sfe:UpdateOrderHallMissionFrame();

			local OldOrderHallMissionFrame_MissionTab_MissionList = missionList.Update;
			function missionList:Update()
				OldOrderHallMissionFrame_MissionTab_MissionList(self);
				Sfe:OrderHallMissionFrame_MissionTab_MissionListPostHook(self);
			end

			if (gmmRunning) then
				eventController:RegisterEvent("GARRISON_MISSION_NPC_OPENED");
			end
		elseif (gmmRunning and event == "GARRISON_MISSION_NPC_OPENED") then
			local missionList = OrderHallMissionFrame.MissionTab.MissionList;

			if (not SfeGmmFix) then
				SfeGmmFix = function()
						if (not missionList or not missionList:IsShown()) then
							return;
						end

						local missions = missionList.showInProgress and missionList.inProgressMissions or missionList.availableMissions;
						local buttons = missionList.listScroll.buttons;
						local offset = HybridScrollFrame_GetOffset(missionList.listScroll);

						for i = 1, #buttons do
							local button = buttons[i];
							local index = offset + i;

							if (not button.GmmButton) then
								for i, c in pairs({button:GetChildren()}) do
									local cPoint, cRelativeTo, cRelativePoint, cXOfs, cYOfs = c:GetPoint(1);
									if (c:GetName() == nil
											and math.floor(c:GetWidth() + 0.5) == 80
											and math.floor(c:GetHeight() + 0.5) == 40
											and cPoint == "LEFT"
											and cRelativeTo == button
											and cRelativePoint == "RIGHT"
											and cXOfs == -65
											and cYOfs == 0
											and c:GetScript("OnClick")) then
										button.GmmButton = c;
										break;
									end
								end
							end

							if (SFE_ORDERHALL_MISSIONS_ENHANCED) then
								if (index <= #missions) then
									local mission = missions[offset + i];

									if (not mission.inProgress) then
										local levelTextY = 0;
										if (mission.isMaxLevel and mission.iLevel > 0) then
											button.Level:SetText(mission.iLevel);
											button.ItemLevel:SetText(ITEM_LEVEL_ABBR);
											button.ItemLevel:Show();
											levelTextY = - button.ItemLevel:GetHeight() / 2;
										else
											button.ItemLevel:Hide();
										end
										button.Level:SetPoint("CENTER", button, "LEFT", 40, levelTextY);
									end

									if (not mission.inProgress) then
										button.Rewards[1]:SetPoint("RIGHT", button, "RIGHT", -12 - 56 * (#mission.rewards + #mission.overmaxRewards - 1) - 60, 0);
									end
								end

								if (button.GmmButton) then
									button.GmmButton:SetWidth(72);
								end
							else
								button.Rewards[1]:SetPoint("RIGHT", button, "RIGHT", - 60, 0);

								if (button.GmmButton) then
									button.GmmButton:SetWidth(80);
								end
							end
						end
					end;
				hooksecurefunc(missionList, "Update", SfeGmmFix);
				hooksecurefunc(missionList.listScroll, "update", SfeGmmFix);
			end

			C_Timer.After(0.1, SfeGmmFix);
		end
	end);
